/*!
 * vue-fancy-player v0.0.8
 * (c) 2020-present sinchang <sinchangwen@gmail.com>
 * Released under the MIT License.
 */
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var DPlayer = _interopDefault(require('fancy-player'));
require('fancy-player/dist/fancy-player.css');

/*
 * @Author: your name
 * @Date: 2020-05-20 16:56:20
 * @LastEditTime: 2020-05-25 11:53:01
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: \vue-dplayer-master\src\index.js
 */
var VueDPlayer = {
  props: {
    options: {
      type: Object
    },
    haveData: {
      type: Boolean
    }
  },
  watch: {
    haveData: function (value) {
      if (value) {
        this.initPlayer();
      }
    }
  },
  data: function data() {
    return {
      dp: null,
      currentIndex: 0,
      videoData: {},
      isFullScreen: false,
      fullScreenType: "web",
      qiniuUrl: 'http://qiniu.tongyishidai.com/',
      imageUrl: '',
      nodes: []
    };
  },
  mounted: function mounted() {
    this.createImgDom();
  },
  methods: {
    initPlayer: function initPlayer() {
      var this$1 = this;

      var highlight = this.options.highlight;
      if (highlight && highlight.length) {
        if (highlight[0].fileType === 1) {
          this.options.video.url = this.qiniuUrl + highlight[0].fileUrl;
          this.options.video.pic = this.qiniuUrl + highlight[0].fileUrl + '?vframe/jpg/offset/0';
        } else if (highlight[0].fileType === 2) {
          this.addClass(this.boxDom, 'show');
          this.imageUrl = this.qiniuUrl + highlight[0].fileUrl;
          this.imgDom.setAttribute('src', this.imageUrl);
        }
      }
      this.options.container = this.$el;
      var player = this.dp = new DPlayer(this.options);
      console.log(this.options);
      var events = player.events;
      this.navigateRight.style.color = player.options.theme;
      this.navigateLeft.style.color = player.options.theme;
      Object.keys(events).forEach(function (item) {
        if (item === 'events') {
          return false;
        } else {
          events[item].forEach(function (event) {
            player.on(event, function () { return this$1.$emit(event); });
          });
        }
      });
      if (!highlight || !highlight.length) {
        return;
      }
      this.handleChangeNode(this.currentIndex);
      player.on('ended', function () {
        if (this$1.currentIndex < highlight.length - 1) {
          this$1.handleChangeNode(this$1.currentIndex + 1);
        }
      });

      player.on("node", function (data) {
        this$1.handleChangeNode(data.index);
      });
      player.on("fullscreen", function () {
        this$1.isFullScreen = true;
        this$1.fullScreenType = "browser";
      });
      player.on("webfullscreen", function () {
        this$1.isFullScreen = true;
        this$1.fullScreenType = "web";
      });
      player.on("webfullscreen_cancel", function () {
        this$1.isFullScreen = false;
      });
      player.on("fullscreen_cancel", function () {
        this$1.isFullScreen = false;
      });
      player.on("durationchange", function () {
        player.section.setHighlights(this$1.nodes);
      });
      player.on("timeupdate", function () {
        var currentTime = Math.floor(player.dom.video.currentTime);
        if (this$1.currentTime !== currentTime) {
          this$1.currentTime = currentTime;
          if (this$1.nodes) {
            for (var i = 0; i < this$1.nodes.length; i++) {
              console.log(currentTime, this$1.nodes[i].time);
              if (currentTime === this$1.nodes[i].time) {
                player.pause();
              }
            }
          }
        }
      });
    },
    createImgDom: function createImgDom() {
      var this$1 = this;

      var boxDom = this.boxDom = document.createElement('div');
      var imgDom = this.imgDom = document.createElement('img');
      var navigateRight = this.navigateRight = document.createElement('button');
      var navigateLeft = this.navigateLeft = document.createElement('button');
      var arrowRight = document.createElement('div');
      var arrowLeft = document.createElement('div');
      navigateRight.className = 'dplayer-navigate-right';
      navigateLeft.className = 'dplayer-navigate-left';
      arrowRight.className = 'dplayer-controls-arrow';
      arrowLeft.className = 'dplayer-controls-arrow';
      boxDom.className = 'dplayer-image-box';
      imgDom.className = 'dplayer-image';
      imgDom.setAttribute('preview', 1);
      imgDom.setAttribute('src', this.imageUrl);
      navigateRight.appendChild(arrowRight);
      navigateLeft.appendChild(arrowLeft);
      boxDom.appendChild(imgDom);
      boxDom.appendChild(navigateRight);
      boxDom.appendChild(navigateLeft);
      navigateRight.addEventListener('click', function () {
        this$1.handleChangeNode(this$1.currentIndex + 1);
      });
      navigateLeft.addEventListener('click', function () {
        this$1.handleChangeNode(this$1.currentIndex - 1);
      });
      this.$el.appendChild(boxDom);
    },
    formatTime: function formatTime(time) {
      var date = new Date(time);
      var hours = date.getHours();
      var minutes = date.getMinutes();
      var seconds = date.getSeconds();
      return 3600 * hours + 60 * minutes + seconds;
    },
    handleChangeNode: function handleChangeNode(index) {
      var this$1 = this;

      var highlight = this.options.highlight;
      if (index < 0 || index >= highlight.length) {
        return;
      }
      this.currentIndex = index;
      this.nodes = [];
      if (highlight[index].fileType === 1) {
        if (this.dp.section.sectionDoms.length) {
          if (highlight[index].nodes && highlight[index].nodes.length) {
            for (var i = 0; i < highlight[index].nodes.length; i++) {
              var nodes = {
                guid: highlight[index].nodes[i].guid,
                time: highlight[index].nodes[i].time
              };
              this$1.nodes.push(nodes);
              this$1.nodes[i].time = this$1.formatTime(this$1.nodes[i].time);
            }
            this.nodes = this.nodes.filter(function (item) { return item.time; });
          }
          this.dp.section.nodesActive_(index, this.dp.options.theme);
          this.videoData.url = this.qiniuUrl + highlight[index].fileUrl;
          this.videoData.pic = this.qiniuUrl + highlight[index].fileUrl + "?vframe/jpg/offset/0";
          this.dp.switchVideo(this.videoData);
          this.dp.play();
          this.removeClass(this.boxDom, 'show');
        }
      } else {
        if (this.isFullScreen) {
          this.dp.fullScreen.toggle(this.fullScreenType);
        }
        if (!this.dp.paused) {
          this.dp.pause();
        }
        if (index === 0) {
          this.addClass(this.navigateLeft, 'lucency');
          this.removeClass(this.navigateRight, 'lucency');
        } else if (index === highlight.length - 1) {
          this.removeClass(this.navigateLeft, 'lucency');
          this.addClass(this.navigateRight, 'lucency');
        } else {
          this.removeClass(this.navigateRight, 'lucency');
          this.removeClass(this.navigateLeft, 'lucency');
        }
        this.addClass(this.boxDom, 'show');
        this.imageUrl = this.qiniuUrl + highlight[index].fileUrl;
        this.imgDom.setAttribute('src', this.imageUrl);
      }
    },
    addClass: function addClass(element, className) {
      if (this.hasClass(element, className)) {
        return;
      }
      var newClass = element.className.split(' ');
      newClass.push(className);
      element.className = newClass.join(' ');
    },
    removeClass: function removeClass(element, className) {
      if (this.hasClass(element, className)) {
        var reg = new RegExp('(\\s|^)' + className + '(\\s|$)');
        element.className = element.className.replace(reg, '');
      }
    },
    hasClass: function hasClass(element, className) {
      var reg = new RegExp('(^|\\s)' + className + '(\\s|$)');
      return reg.test(element.className);
    }
  },
  install: function install(Vue, ref) {
    if ( ref === void 0 ) ref = {};
    var name = ref.name; if ( name === void 0 ) name = 'd-player';

    Vue.component(name, this);
  },
  render: function render(h) {
    return h('div', {
      class: 'dplayer'
    }, []);
  }
};

if (typeof window !== 'undefined' && window.Vue) {
  window.VueDPlayer = VueDPlayer;
}

module.exports = VueDPlayer;
