/*!
 * vue-fancy-player v0.0.8
 * (c) 2020-present sinchang <sinchangwen@gmail.com>
 * Released under the MIT License.
 */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.vueFancyPlayer = factory());
}(this, (function () { 'use strict';

var handleOption = (function (options) {
    // default options
    var defaultOption = {
        container: options.container || document.getElementsByClassName('fancy-player')[0],
        live: false,
        autoplay: false,
        theme: '#4588bf',
        loop: false,
        lang: 'zh-cn',
        screenshot: false,
        hotkey: true,
        preload: 'metadata',
        volume: 0.7,
        apiBackend: 'defaultApiBackend',
        video: { url: '' },
        mutex: true
    };
    for (var defaultKey in defaultOption) {
        if (defaultOption.hasOwnProperty(defaultKey) && !options.hasOwnProperty(defaultKey)) {
            options[defaultKey] = defaultOption[defaultKey];
        }
    }
    if (options.video) {
        !options.video.type && (options.video.type = 'auto');
    }
    if (typeof options.danmaku === 'object' && options.danmaku) {
        !options.danmaku.user && (options.danmaku.user = 'DIYgod');
    }
    if (options.subtitle) {
        !options.subtitle.type && (options.subtitle.type = 'webvtt');
        !options.subtitle.fontSize && (options.subtitle.fontSize = '20px');
        !options.subtitle.bottom && (options.subtitle.bottom = '40px');
        !options.subtitle.color && (options.subtitle.color = '#fff');
    }
    if (options.video.quality) {
        options.video.url = options.video.quality[options.video.defaultQuality].url;
    }
    if (options.highlight && options.highlight.length) {
        options.isTimeNode = options.highlight.some(function (item) { return typeof item.time !== 'undefined'; });
    }
    return options;
});

// add translation text here
var tranTxt = {
    'zh-cn': {
        'Danmaku is loading': '弹幕加载中',
        Top: '顶部',
        Bottom: '底部',
        Rolling: '滚动',
        'Input danmaku, hit Enter': '输入弹幕，回车发送',
        'About author': '关于作者',
        'DPlayer feedback': '播放器意见反馈',
        'About DPlayer': '关于 DPlayer 播放器',
        Loop: '洗脑循环',
        Speed: '速度',
        'Opacity for danmaku': '弹幕透明度',
        Normal: '正常',
        'Please input danmaku content!': '要输入弹幕内容啊喂！',
        'Set danmaku color': '设置弹幕颜色',
        'Set danmaku type': '设置弹幕类型',
        'Show danmaku': '显示弹幕',
        'Video load failed': '视频加载失败',
        'Danmaku load failed': '弹幕加载失败',
        'Danmaku send failed': '弹幕发送失败',
        'Switching to': '正在切换至',
        'Switched to': '已经切换至',
        quality: '画质',
        FF: '快进',
        REW: '快退',
        'Unlimited danmaku': '海量弹幕',
        'Send danmaku': '发送弹幕',
        Setting: '设置',
        'Full screen': '全屏',
        'Web full screen': '页面全屏',
        Send: '发送',
        Screenshot: '截图',
        s: '秒',
        'Show subtitle': '显示字幕',
        'Hide subtitle': '隐藏字幕',
        Volume: '音量',
        Live: '直播',
        'Video info': '视频统计信息'
    },
    'zh-tw': {
        'Danmaku is loading': '彈幕載入中',
        Top: '頂部',
        Bottom: '底部',
        Rolling: '滾動',
        'Input danmaku, hit Enter': '輸入彈幕，Enter 發送',
        'About author': '關於作者',
        'DPlayer feedback': '播放器意見回饋',
        'About DPlayer': '關於 DPlayer 播放器',
        Loop: '循環播放',
        Speed: '速度',
        'Opacity for danmaku': '彈幕透明度',
        Normal: '正常',
        'Please input danmaku content!': '請輸入彈幕內容啊！',
        'Set danmaku color': '設定彈幕顏色',
        'Set danmaku type': '設定彈幕類型',
        'Show danmaku': '顯示彈幕',
        'Video load failed': '影片載入失敗',
        'Danmaku load failed': '彈幕載入失敗',
        'Danmaku send failed': '彈幕發送失敗',
        'Switching to': '正在切換至',
        'Switched to': '已經切換至',
        quality: '畫質',
        FF: '快進',
        REW: '快退',
        'Unlimited danmaku': '巨量彈幕',
        'Send danmaku': '發送彈幕',
        Setting: '設定',
        'Full screen': '全螢幕',
        'Web full screen': '頁面全螢幕',
        Send: '發送',
        Screenshot: '截圖',
        s: '秒',
        'Show subtitle': '顯示字幕',
        'Hide subtitle': '隱藏字幕',
        Volume: '音量',
        Live: '直播',
        'Video info': '影片統計訊息'
    }
};
var I18n = /** @class */ (function () {
    function I18n(lang) {
        this.lang = lang;
        this.tran = this.tran.bind(this);
    }
    I18n.prototype.tran = function (text) {
        if (tranTxt[this.lang] && tranTxt[this.lang][text]) {
            return tranTxt[this.lang][text];
        }
        else {
            return text;
        }
    };
    return I18n;
}());

var Events = /** @class */ (function () {
    function Events() {
        this.events = {};
        this.videoEvents = [
            'abort',
            'canplay',
            'canplaythrough',
            'durationchange',
            'emptied',
            'ended',
            'error',
            'loadeddata',
            'loadedmetadata',
            'loadstart',
            'mozaudioavailable',
            'pause',
            'play',
            'playing',
            'progress',
            'ratechange',
            'seeked',
            'seeking',
            'stalled',
            'suspend',
            'timeupdate',
            'volumechange',
            'waiting'
        ];
        this.playerEvents = [
            'screenshot',
            'thumbnails_show',
            'thumbnails_hide',
            'danmaku_show',
            'danmaku_hide',
            'danmaku_clear',
            'danmaku_loaded',
            'danmaku_send',
            'danmaku_opacity',
            'contextmenu_show',
            'contextmenu_hide',
            'notice_show',
            'notice_hide',
            'quality_start',
            'quality_end',
            'destroy',
            'resize',
            'fullscreen',
            'fullscreen_cancel',
            'webfullscreen',
            'webfullscreen_cancel',
            'subtitle_show',
            'subtitle_hide',
            'subtitle_change',
            'node'
        ];
    }
    Events.prototype.on = function (name, callback) {
        if (this.type(name) && typeof callback === 'function') {
            if (!this.events[name]) {
                this.events[name] = [];
            }
            this.events[name].push(callback);
        }
    };
    Events.prototype.trigger = function (name, info) {
        if (this.events[name] && this.events[name].length) {
            for (var i = 0; i < this.events[name].length; i++) {
                this.events[name][i](info);
            }
        }
    };
    Events.prototype.type = function (name) {
        if (this.playerEvents.indexOf(name) !== -1) {
            return 'player';
        }
        else if (this.videoEvents.indexOf(name) !== -1) {
            return 'video';
        }
        console.error("Unknown event name: " + name);
        return null;
    };
    return Events;
}());

var _manualClassList = new (/** @class */ (function () {
    function class_1() {
    }
    class_1.prototype._findClassName = function (node, className) {
        var classes = node.className;
        if (!classes) {
            this._lastStart = -1;
            return;
        }
        className = className.trim();
        var classesLen = classes.length;
        var classLen = className.length;
        if (classLen === 0) {
            this._lastStart = -1;
            return;
        }
        if (classesLen < classLen) {
            this._lastStart = -1;
            return;
        }
        if (classes === className) {
            this._lastStart = 0;
            this._lastEnd = classesLen;
            return;
        }
        var idx = -1;
        var idxEnd;
        // tslint:disable-next-line
        while ((idx = classes.indexOf(className, idx + 1)) >= 0) {
            idxEnd = idx + classLen;
            // a class that is followed by another class
            if ((idx === 0 || classes.charCodeAt(idx - 1) === 32 /* Space */) &&
                classes.charCodeAt(idxEnd) === 32 /* Space */) {
                this._lastStart = idx;
                this._lastEnd = idxEnd + 1;
                return;
            }
            // last class
            if (idx > 0 && classes.charCodeAt(idx - 1) === 32 /* Space */ && idxEnd === classesLen) {
                this._lastStart = idx - 1;
                this._lastEnd = idxEnd;
                return;
            }
            // equal - duplicate of cmp above
            if (idx === 0 && idxEnd === classesLen) {
                this._lastStart = 0;
                this._lastEnd = idxEnd;
                return;
            }
        }
        this._lastStart = -1;
    };
    class_1.prototype.hasClass = function (node, className) {
        this._findClassName(node, className);
        return this._lastStart !== -1;
    };
    class_1.prototype.addClasses = function (node) {
        var _this = this;
        var classNames = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            classNames[_i - 1] = arguments[_i];
        }
        classNames.forEach(function (nameValue) { return nameValue.split(' ').forEach(function (name) { return _this.addClass(node, name); }); });
    };
    class_1.prototype.addClass = function (node, className) {
        if (!node.className) {
            // doesn't have it for sure
            node.className = className;
        }
        else {
            this._findClassName(node, className); // see if it's already there
            if (this._lastStart === -1) {
                node.className = node.className + ' ' + className;
            }
        }
    };
    class_1.prototype.removeClass = function (node, className) {
        this._findClassName(node, className);
        if (this._lastStart === -1) {
            return; // Prevent styles invalidation if not necessary
        }
        else {
            node.className =
                node.className.substring(0, this._lastStart) + node.className.substring(this._lastEnd);
        }
    };
    class_1.prototype.removeClasses = function (node) {
        var _this = this;
        var classNames = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            classNames[_i - 1] = arguments[_i];
        }
        classNames.forEach(function (nameValue) {
            return nameValue.split(' ').forEach(function (name) { return _this.removeClass(node, name); });
        });
    };
    class_1.prototype.toggleClass = function (node, className, shouldHaveIt) {
        this._findClassName(node, className);
        if (this._lastStart !== -1 && (shouldHaveIt === undefined || !shouldHaveIt)) {
            this.removeClass(node, className);
        }
        if (this._lastStart === -1 && (shouldHaveIt === undefined || shouldHaveIt)) {
            this.addClass(node, className);
        }
    };
    return class_1;
}()))();
var _nativeClassList = new (/** @class */ (function () {
    function class_2() {
    }
    class_2.prototype.hasClass = function (node, className) {
        return Boolean(className) && node.classList && node.classList.contains(className);
    };
    class_2.prototype.addClasses = function (node) {
        var _this = this;
        var classNames = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            classNames[_i - 1] = arguments[_i];
        }
        classNames.forEach(function (nameValue) { return nameValue.split(' ').forEach(function (name) { return _this.addClass(node, name); }); });
    };
    class_2.prototype.addClass = function (node, className) {
        if (className && node.classList) {
            node.classList.add(className);
        }
    };
    class_2.prototype.removeClass = function (node, className) {
        if (className && node.classList) {
            node.classList.remove(className);
        }
    };
    class_2.prototype.removeClasses = function (node) {
        var _this = this;
        var classNames = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            classNames[_i - 1] = arguments[_i];
        }
        classNames.forEach(function (nameValue) {
            return nameValue.split(' ').forEach(function (name) { return _this.removeClass(node, name); });
        });
    };
    class_2.prototype.toggleClass = function (node, className, shouldHaveIt) {
        if (node.classList) {
            node.classList.toggle(className, shouldHaveIt);
        }
    };
    return class_2;
}()))();
var Utils = /** @class */ (function () {
    function Utils() {
    }
    /**
     * Parse second to time string
     *
     * @param {Number} second
     * @return {String} 00:00 or 00:00:00
     */
    Utils.secondToTime = function (second) {
        var add0 = function (num) { return (num < 10 ? '0' + num : '' + num); };
        var hour = Math.floor(second / 3600);
        var min = Math.floor((second - hour * 3600) / 60);
        var sec = Math.floor(second - hour * 3600 - min * 60);
        return (hour > 0 ? [hour, min, sec] : [min, sec]).map(add0).join(':');
    };
    Utils.getScrollPosition = function () {
        return {
            left: window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft || 0,
            top: window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0
        };
    };
    Utils.setScrollPosition = function (lastScrollPosition) {
        var _a = lastScrollPosition.left, left = _a === void 0 ? 0 : _a, _b = lastScrollPosition.top, top = _b === void 0 ? 0 : _b;
        if (Utils.isFirefox) {
            document.documentElement.scrollLeft = left;
            document.documentElement.scrollTop = top;
        }
        else {
            window.scrollTo(left, top);
        }
    };
    Utils.getBoundingClientRectViewLeftOrTop = function (element, direction) {
        var scrollTop = window.scrollY ||
            window.pageYOffset ||
            document.body.scrollTop +
                ((document.documentElement && document.documentElement.scrollTop) || 0);
        if (element.getBoundingClientRect) {
            if (typeof Utils.offset !== 'number') {
                var temp = document.createElement('div');
                temp.style.cssText = 'position:absolute;top:0;left:0;';
                document.body.appendChild(temp);
                Utils.offset = -temp.getBoundingClientRect().top - scrollTop;
                document.body.removeChild(temp);
                temp = null;
            }
            var rect = element.getBoundingClientRect();
            var offset = Utils.offset;
            return rect[direction] + offset;
        }
        else {
            // not support getBoundingClientRect
            return Utils.getElementViewLeft(element);
        }
    };
    Utils.getElementViewLeft = function (element) {
        var actualLeft = element.offsetLeft;
        var current = element.offsetParent;
        var elementScrollLeft = document.body.scrollLeft + document.documentElement.scrollLeft;
        if (!document.fullscreenElement &&
            !document.mozFullScreenElement &&
            !document.webkitFullscreenElement) {
            while (current !== null) {
                actualLeft += current.offsetLeft;
                current = current.offsetParent;
            }
        }
        else {
            while (current !== null && current !== element) {
                actualLeft += current.offsetLeft;
                current = current.offsetParent;
            }
        }
        return actualLeft - elementScrollLeft;
    };
    Utils.cumulativeOffset = function (element) {
        var top = 0;
        var left = 0;
        do {
            top += element.offsetTop || 0;
            left += element.offsetLeft || 0;
            element = element.offsetParent;
        } while (element);
        return {
            top: top,
            left: left
        };
    };
    Utils.number2Color = function (num) {
        return '#' + ('00000' + num.toString(16)).slice(-6);
    };
    Utils.number2Type = function (num) {
        switch (num) {
            case 0:
                return 'right';
            case 1:
                return 'top';
            case 2:
                return 'bottom';
            default:
                return 'right';
        }
    };
    Utils.isIE = navigator.userAgent.indexOf('Trident') >= 0;
    Utils.isChrome = navigator.userAgent.indexOf('Chrome') >= 0;
    Utils.isFirefox = navigator.userAgent.indexOf('Firefox') >= 0;
    Utils.isMobile = navigator.userAgent.indexOf('Android') >= 0 ||
        navigator.userAgent.indexOf('Adr') >= 0 ||
        !!navigator.userAgent.match(/\(i[^;]+;( U;)? CPU.+Mac OS X/);
    Utils.storage = {
        set: function (key, value) {
            localStorage.setItem(key, value);
        },
        get: function (key) {
            return localStorage.getItem(key);
        }
    };
    Utils.classList = Utils.isIE ? _manualClassList : _nativeClassList;
    Utils.nameMap = {
        dragStart: Utils.isMobile ? 'touchstart' : 'mousedown',
        dragMove: Utils.isMobile ? 'touchmove' : 'mousemove',
        dragEnd: Utils.isMobile ? 'touchend' : 'mouseup'
    };
    return Utils;
}());

var User = /** @class */ (function () {
    function User(player) {
        this.storageName = {
            opacity: 'dplayer-danmaku-opacity',
            volume: 'dplayer-volume',
            unlimited: 'dplayer-danmaku-unlimited',
            danmaku: 'dplayer-danmaku-show',
            subtitle: 'dplayer-subtitle-show'
        };
        this.default = {
            opacity: 0.7,
            volume: player.options.hasOwnProperty('volume') ? player.options.volume : 0.7,
            unlimited: (player.options.danmaku && player.options.danmaku.unlimited ? 1 : 0) || 0,
            danmaku: 1,
            subtitle: 1
        };
        this.data = {};
        this.init();
    }
    User.prototype.init = function () {
        for (var item in this.storageName) {
            var name_1 = this.storageName[item];
            this.data[item] = parseFloat(Utils.storage.get(name_1) || this.default[item]);
        }
    };
    User.prototype.get = function (key) {
        return this.data[key];
    };
    User.prototype.set = function (key, value) {
        this.data[key] = value;
        Utils.storage.set(this.storageName[key], value);
    };
    return User;
}());

var Dom = /** @class */ (function () {
    function Dom(element, options, tran) {
        this.container = element;
        this.init(element, options, tran);
    }
    Dom.prototype.init = function (element, options, tran) {
        var video = options.video, preload = options.preload, screenshot = options.screenshot, logo = options.logo, theme = options.theme, danmaku = options.danmaku, programs = options.programs, highlight = options.highlight;
        this.mask = this.createDom({ ele: element, label: 'div', className: 'dplayer-mask' });
        this.videoWrap = this.createDom({ ele: element, label: 'div', className: 'dplayer-video-wrap' });
        // 视频
        if (video) {
            var _a = video.url, url = _a === void 0 ? '' : _a, _b = video.pic, pic = _b === void 0 ? '' : _b;
            this.video = this.createMediaDom({
                ele: this.videoWrap,
                label: 'video',
                className: 'dplayer-video dplayer-video-current',
                url: url,
                poster: pic,
                preload: preload,
                screenshot: screenshot,
                playsinline: 'true',
                webkitPlaysinline: 'true'
            });
        }
        if (logo) {
            var playerLogo = this.createDom({
                ele: this.videoWrap,
                label: 'div',
                className: 'dplayer-logo'
            });
            this.createDom({ ele: playerLogo, label: 'img', url: logo });
        }
        if ((programs && programs.length > 1) || (highlight && highlight.length)) {
            this.sidebar = this.createDom({
                ele: element,
                label: 'div',
                className: 'dplayer-sidebar'
            });
            if (programs && programs.length > 1) {
                var program = this.createDom({
                    ele: this.sidebar,
                    label: 'div',
                    className: 'dplayer-program'
                });
                this.createDom({
                    ele: program,
                    label: 'div',
                    className: 'dplayer-program-item',
                    styles: "background-image: url(" + programs[0] + ")"
                });
                this.createDom({
                    ele: program,
                    label: 'div',
                    className: 'dplayer-program-item',
                    styles: "background-image: url(" + programs[1] + ")"
                });
            }
            if (highlight && highlight.length) {
                this.section = this.createDom({
                    ele: this.sidebar,
                    label: 'div',
                    className: 'dplayer-section'
                });
                this.sectionInner = this.createDom({
                    ele: this.section,
                    label: 'div',
                    className: 'dplayer-section-inner'
                });
            }
        }
        if (danmaku) {
            this.danmaku = this.createDom({
                ele: this.videoWrap,
                label: 'div',
                className: 'dplayer-danmaku'
            });
            this.createDom({
                ele: this.danmaku,
                label: 'div',
                className: 'dplayer-danmaku-item dplayer-danmaku-item--demo'
            });
        }
        var bezelWrap = this.createDom({
            ele: this.videoWrap,
            label: 'div',
            className: 'dplayer-bezel'
        });
        this.bezelIconBox = this.createDom({
            ele: bezelWrap,
            label: 'span',
            className: 'dplayer-bezel-icon dplayer-bezel-pause'
        });
        var LoadHtmlStr = "<div class=\"spinner1\"><div class=\"bounce1\"></div><div class=\"bounce2\"></div><div class=\"bounce3\"></div></div>";
        this.loadingIcon = this.createDom({
            ele: bezelWrap,
            label: 'span',
            className: 'diplayer-loading-icon',
            html: LoadHtmlStr
        });
        this.bezelIcon = this.createDom({
            ele: this.bezelIconBox,
            label: 'span',
            className: 'iconfont iconplay'
        });
        this.controllerMask = this.createDom({
            ele: element,
            label: 'div',
            className: 'dplayer-controller-mask'
        });
        this.controller = this.createDom({
            ele: element,
            label: 'div',
            className: 'dplayer-controller'
        });
        this.controllerLeft = this.createDom({
            ele: this.controller,
            label: 'div',
            className: 'dplayer-icons dplayer-icons-left'
        });
        this.playRetreat = this.createDom({
            ele: this.controllerLeft,
            label: 'button',
            className: 'dplayer-icon dplayer-retreat'
        });
        this.createDom({
            ele: this.playRetreat,
            label: 'span',
            className: 'iconfont iconretreat'
        });
        this.playButton = this.createDom({
            ele: this.controllerLeft,
            label: 'button',
            className: 'dplayer-icon dplayer-play-icon'
        });
        this.playButtonIco = this.createDom({
            ele: this.playButton,
            label: 'span',
            className: 'iconfont iconplay'
        });
        this.playFast = this.createDom({
            ele: this.controllerLeft,
            label: 'button',
            className: 'dplayer-icon dplayer-fast'
        });
        this.createDom({
            ele: this.playFast,
            label: 'span',
            className: 'iconfont iconfast'
        });
        // 控制右侧
        var controllerRight = this.createDom({
            ele: this.controller,
            label: 'div',
            className: 'dplayer-icons dplayer-icons-right'
        });
        var playerTime = this.createDom({
            ele: controllerRight,
            label: 'span',
            className: 'dplayer-time'
        });
        this.ptime = this.createDom({
            ele: playerTime,
            label: 'span',
            className: 'dplayer-ptime',
            text: '0:00'
        });
        this.createDom({ ele: playerTime, label: 'span', text: ' / ' });
        this.dtime = this.createDom({
            ele: playerTime,
            label: 'span',
            className: 'dplayer-dtime',
            text: '0:00'
        });
        var doubleSpeed = this.createDom({
            ele: controllerRight,
            label: 'div',
            className: 'dplayer-double-speed-wrap'
        });
        this.doubleSpeed = this.createDom({
            ele: doubleSpeed,
            text: '倍速',
            label: 'span',
            className: 'dplayer-double-speed'
        });
        var doubleSpeedStr = "<div class=\"dplayer-popup-speed-item\" data-speed=\"0.5X\">0.5X</div>\n    <div class=\"dplayer-popup-speed-item\" data-speed=\"1X\">1X</div>\n    <div class=\"dplayer-popup-speed-item\" data-speed=\"1.25X\">1.25X</div>\n    <div class=\"dplayer-popup-speed-item\" data-speed=\"1.5X\">1.5X</div>\n    <div class=\"dplayer-popup-speed-item\" data-speed=\"2X\">2X</div>\n    ";
        this.doubleSpeedPopup = this.createDom({
            ele: doubleSpeed,
            label: 'div',
            html: doubleSpeedStr,
            className: 'dplayer-popup-panel'
        });
        this.volumeButton = this.createDom({
            ele: controllerRight,
            label: 'div',
            className: 'dplayer-volume'
        });
        this.volumeButtonIcon = this.createDom({
            ele: this.volumeButton,
            label: 'button',
            className: 'dplayer-icon dplayer-volume-icon'
        });
        this.volumeIcon = this.createDom({
            ele: this.volumeButtonIcon,
            label: 'span',
            className: 'dplayer-icon-content'
        });
        this.volumeIconFont = this.createDom({
            ele: this.volumeIcon,
            label: 'span',
            className: 'iconfont iconvolume-up'
        });
        this.volumeBarWrapWrap = this.createDom({
            ele: this.volumeButton,
            label: 'div',
            className: 'dplayer-volume-bar-wrap'
        });
        // this.volumeBarWrapWrap.setAttribute('data-balloon-pos', 'up')
        this.volumeBarWrap = this.createDom({
            ele: this.volumeBarWrapWrap,
            label: 'div',
            className: 'dplayer-volume-bar'
        });
        this.volumeBar = this.createDom({
            ele: this.volumeBarWrap,
            label: 'div',
            className: 'dplayer-volume-bar-inner',
            styles: "background:" + theme
        });
        this.createDom({
            ele: this.volumeBar,
            label: 'span',
            className: 'dplayer-thumb',
            styles: "background:" + theme
        });
        // 全屏按钮
        var playerFull = this.createDom({
            ele: controllerRight,
            label: 'div',
            className: 'dplayer-full'
        });
        this.webFullButton = this.createDom({
            ele: playerFull,
            label: 'button',
            className: 'dplayer-icon dplayer-full-in-icon'
        });
        this.browserFullButton = this.createDom({
            ele: playerFull,
            label: 'button',
            className: 'dplayer-icon dplayer-full-icon'
        });
        var browserFullButIco = this.createDom({
            ele: this.browserFullButton,
            label: 'span',
            className: 'dplayer-icon-content'
        });
        var webFullButIco = this.createDom({
            ele: this.webFullButton,
            label: 'span',
            className: 'dplayer-icon-content'
        });
        this.createDom({ ele: browserFullButIco, label: 'span', className: 'iconfont iconfull' });
        this.createDom({ ele: webFullButIco, label: 'span', className: 'iconfont iconfull-web' });
        this.playedBarWrap = this.createDom({
            ele: this.controller,
            label: 'div',
            className: 'dplayer-bar-wrap'
        });
        this.playedBarTime = this.createDom({
            ele: this.playedBarWrap,
            label: 'div',
            className: 'dplayer-bar-time hidden',
            text: '00:00'
        });
        this.playedBarPreview = this.createDom({
            ele: this.playedBarWrap,
            label: 'div',
            className: 'dplayer-bar-preview'
        });
        var playerBar = this.createDom({
            ele: this.playedBarWrap,
            label: 'div',
            className: 'dplayer-bar'
        });
        this.loadedBar = this.createDom({
            ele: playerBar,
            label: 'div',
            className: 'dplayer-loaded',
            styles: 'width: 0'
        });
        this.playedBar = this.createDom({
            ele: playerBar,
            label: 'div',
            className: 'dplayer-played',
            styles: "width: 0; background: " + theme
        });
        this.createDom({
            ele: this.playedBar,
            label: 'span',
            className: 'dplayer-thumb',
            styles: "background: " + theme
        });
        this.notice = this.createDom({ ele: element, label: 'div', className: 'dplayer-notice' });
    };
    Dom.prototype.createMediaDom = function (params) {
        var label = params.label, className = params.className, ele = params.ele, url = params.url, poster = params.poster, preload = params.preload, screenshot = params.screenshot, playsinline = params.playsinline, webkitPlaysinline = params.webkitPlaysinline;
        var createEle = document.createElement(label);
        if (className) {
            createEle.className = className;
        }
        if (url) {
            createEle.setAttribute('src', url);
        }
        if (preload) {
            createEle.setAttribute('preload', preload);
        }
        if (poster) {
            createEle.setAttribute('poster', poster);
        }
        if (playsinline) {
            createEle.setAttribute('playsinline', playsinline);
        }
        if (webkitPlaysinline) {
            createEle.setAttribute('webkit-playsinline', webkitPlaysinline);
        }
        if (screenshot) {
            createEle.setAttribute('crossorigin', 'anonymous');
        }
        ele.appendChild(createEle);
        return createEle;
    };
    Dom.prototype.createDom = function (params) {
        var label = params.label, className = params.className, ele = params.ele, url = params.url, styles = params.styles, text = params.text, html = params.html;
        var createEle = document.createElement(label);
        if (className) {
            createEle.className = className;
        }
        if (url) {
            createEle.setAttribute('src', url);
        }
        if (styles) {
            createEle.setAttribute('style', styles);
        }
        if (text) {
            createEle.innerText = text;
        }
        if (html) {
            createEle.innerHTML = html;
        }
        ele.appendChild(createEle);
        return createEle;
    };
    return Dom;
}());

var Bar = /** @class */ (function () {
    function Bar(dom) {
        this.elements = {
            volume: dom.volumeBar,
            played: dom.playedBar,
            loaded: dom.loadedBar
        };
    }
    /**
     * Update progress
     *
     * @param {String} type - Point out which bar it is
     * @param {Number} percentage
     * @param {String} direction - Point out the direction of this bar, Should be height or width
     */
    Bar.prototype.set = function (type, percentage, direction) {
        percentage = Math.max(percentage, 0);
        percentage = Math.min(percentage, 1);
        this.elements[type].style[direction] = percentage * 100 + '%';
    };
    Bar.prototype.get = function (type, direction) {
        direction = direction || 'width';
        return parseFloat(this.elements[type].style[direction]) / 100;
    };
    return Bar;
}());

var FullScreen = /** @class */ (function () {
    function FullScreen(player) {
        var _this = this;
        this.player = player;
        this.player.events.on('webfullscreen', function () {
            _this.player.resize();
        });
        this.player.events.on('webfullscreen_cancel', function () {
            _this.player.resize();
            Utils.setScrollPosition(_this.lastScrollPosition);
        });
        var fullscreenchange = function () {
            _this.player.resize();
            if (_this.isFullScreen('browser')) {
                _this.player.events.trigger('fullscreen');
            }
            else {
                Utils.setScrollPosition(_this.lastScrollPosition);
                _this.player.events.trigger('fullscreen_cancel');
            }
        };
        var docfullscreenchange = function () {
            var fullEle = document.fullscreenElement ||
                document.mozFullScreenElement ||
                document.msFullscreenElement;
            if (fullEle && fullEle !== _this.player.container) {
                return;
            }
            _this.player.resize();
            if (fullEle) {
                _this.player.events.trigger('fullscreen');
            }
            else {
                Utils.setScrollPosition(_this.lastScrollPosition);
                _this.player.events.trigger('fullscreen_cancel');
            }
        };
        if (/Firefox/.test(navigator.userAgent)) {
            document.addEventListener('mozfullscreenchange', docfullscreenchange);
            document.addEventListener('fullscreenchange', docfullscreenchange);
        }
        else {
            this.player.container.addEventListener('fullscreenchange', fullscreenchange);
            this.player.container.addEventListener('webkitfullscreenchange', fullscreenchange);
            document.addEventListener('msfullscreenchange', docfullscreenchange);
            document.addEventListener('MSFullscreenChange', docfullscreenchange);
        }
    }
    FullScreen.prototype.isFullScreen = function (type) {
        if (type === void 0) { type = 'browser'; }
        switch (type) {
            case 'browser':
                return (document.fullscreenElement ||
                    document.mozFullScreenElement ||
                    document.webkitFullscreenElement ||
                    document.msFullscreenElement);
            case 'web':
                return Utils.classList.hasClass(this.player.container, 'dplayer-fulled');
        }
    };
    FullScreen.prototype.request = function (type) {
        if (type === void 0) { type = 'browser'; }
        var anotherType = type === 'browser' ? 'web' : 'browser';
        var anotherTypeOn = this.isFullScreen(anotherType);
        if (!anotherTypeOn) {
            this.lastScrollPosition = Utils.getScrollPosition();
        }
        switch (type) {
            case 'browser':
                if (this.player.container.requestFullscreen) {
                    this.player.container.requestFullscreen();
                }
                else if (this.player.container.mozRequestFullScreen) {
                    this.player.container.mozRequestFullScreen();
                }
                else if (this.player.container.webkitRequestFullscreen) {
                    this.player.container.webkitRequestFullscreen();
                }
                else if (this.player.video.webkitEnterFullscreen) {
                    this.player.video.webkitEnterFullscreen();
                }
                else if (this.player.video.webkitEnterFullScreen) {
                    this.player.video.webkitEnterFullScreen();
                }
                else if (this.player.container.msRequestFullscreen) {
                    this.player.container.msRequestFullscreen();
                }
                break;
            case 'web':
                Utils.classList.addClass(this.player.container, 'dplayer-fulled');
                Utils.classList.addClass(document.body, 'dplayer-web-fullscreen-fix');
                this.player.events.trigger('webfullscreen');
                break;
        }
        if (anotherTypeOn) {
            this.cancel(anotherType);
        }
    };
    FullScreen.prototype.cancel = function (type) {
        if (type === void 0) { type = 'browser'; }
        switch (type) {
            case 'browser':
                if (document.cancelFullScreen) {
                    document.cancelFullScreen();
                }
                else if (document.mozCancelFullScreen) {
                    document.mozCancelFullScreen();
                }
                else if (document.webkitCancelFullScreen) {
                    document.webkitCancelFullScreen();
                }
                else if (document.webkitCancelFullscreen) {
                    document.webkitCancelFullscreen();
                }
                else if (document.msCancelFullScreen) {
                    document.msCancelFullScreen();
                }
                else if (document.msExitFullscreen) {
                    document.msExitFullscreen();
                }
                break;
            case 'web':
                Utils.classList.removeClass(this.player.container, 'dplayer-fulled');
                Utils.classList.removeClass(document.body, 'dplayer-web-fullscreen-fix');
                this.player.events.trigger('webfullscreen_cancel');
                break;
        }
    };
    FullScreen.prototype.toggle = function (type) {
        // setTimeout(() => {
        //   if (this.player.section.getMinDistance() > 0) {
        //     Utils.classList.addClass(this.player.dom.section, 'align-center')
        //   } else {
        //     Utils.classList.removeClass(this.player.dom.section, 'align-center')
        //   }
        // }, 100)
        if (type === void 0) { type = 'browser'; }
        if (this.isFullScreen(type)) {
            this.cancel(type);
        }
        else {
            this.request(type);
        }
    };
    return FullScreen;
}());

var Thumbnails = /** @class */ (function () {
    function Thumbnails(options) {
        this.container = options.container;
        this.barWidth = options.barWidth;
        this.container.style.backgroundImage = "url('" + options.url + "')";
        this.events = options.events;
    }
    Thumbnails.prototype.resize = function (width, height, barWrapWidth) {
        this.container.style.width = width + "px";
        this.container.style.height = height + "px";
        this.container.style.top = -height + 2 + "px";
        this.barWidth = barWrapWidth;
    };
    Thumbnails.prototype.show = function () {
        this.container.style.display = 'block';
        this.events && this.events.trigger('thumbnails_show');
    };
    Thumbnails.prototype.move = function (position) {
        this.container.style.backgroundPosition = "-" + (Math.ceil((position / this.barWidth) * 100) -
            1) *
            160 + "px 0";
        this.container.style.left = Math.min(Math.max(position - this.container.offsetWidth / 2, -10), this.barWidth - 150) + "px";
    };
    Thumbnails.prototype.hide = function () {
        this.container.style.display = 'none';
        this.events && this.events.trigger('thumbnails_hide');
    };
    return Thumbnails;
}());

var Controller = /** @class */ (function () {
    function Controller(player) {
        var _this = this;
        this.player = player;
        this.autoHideTimer = 0;
        this.status = false;
        this.onOff = true;
        this.player.dom.mask.addEventListener('click', function () {
            _this.maskhide();
        });
        if (!Utils.isMobile) {
            this.player.container.addEventListener('mousemove', function () {
                _this.show();
                _this.setAutoHide();
            });
            this.player.container.addEventListener('click', function () {
                _this.setAutoHide();
            });
            this.player.on('play', function () {
                _this.setAutoHide();
            });
            this.player.on('pause', function () {
                _this.setAutoHide();
            });
        }
        this.initPlayButton();
        this.initRetreatAndFast();
        this.initThumbnails();
        this.initPlayedBar();
        this.initFullButton();
        this.initDoubleSpeed();
        // this.initHighlights()
        this.initVolumeButton();
    }
    // 播放按钮
    Controller.prototype.initPlayButton = function () {
        var _this = this;
        this.player.dom.playButton.addEventListener('click', function () {
            _this.player.toggle();
        });
        if (!Utils.isMobile) {
            this.player.dom.videoWrap.addEventListener('click', function () {
                _this.player.toggle();
            });
            this.player.dom.controllerMask.addEventListener('click', function () {
                _this.player.toggle();
            });
        }
        else {
            this.player.dom.videoWrap.addEventListener('click', function () {
                _this.setAutoHide();
                if (_this.status) {
                    _this.status = false;
                    setTimeout(function () {
                        if (!_this.status && _this.onOff) {
                            _this.toggle();
                        }
                        else if (_this.status) {
                            _this.player.toggle();
                            _this.onOff = false;
                        }
                        else if (!_this.onOff) {
                            _this.onOff = true;
                        }
                    }, 300);
                }
                else if (!_this.status) {
                    _this.status = true;
                    setTimeout(function () {
                        if (_this.status && _this.onOff) {
                            _this.toggle();
                        }
                        else if (!_this.status) {
                            _this.player.toggle();
                            _this.onOff = false;
                        }
                        else if (!_this.onOff) {
                            _this.onOff = true;
                        }
                    }, 300);
                }
            });
            this.player.dom.controllerMask.addEventListener('click', function () {
                _this.toggle();
            });
        }
    };
    Controller.prototype.initRetreatAndFast = function () {
        var _this = this;
        this.player.dom.playRetreat.addEventListener('click', function () {
            _this.player.seek(_this.player.video.currentTime - 15);
            _this.player.controller.setAutoHide();
        });
        this.player.dom.playFast.addEventListener('click', function () {
            _this.player.seek(_this.player.video.currentTime + 15);
            _this.player.controller.setAutoHide();
        });
    };
    Controller.prototype.initDoubleSpeed = function () {
        var _this = this;
        var doubleSpeeds = document.querySelectorAll('.dplayer-popup-speed-item');
        this.player.dom.doubleSpeed.addEventListener('click', function () {
            var rateText = _this.player.dom.doubleSpeed.innerText;
            for (var i = 0; i < doubleSpeeds.length; i++) {
                var doubleSpeed = doubleSpeeds[i];
                var rate = doubleSpeed.dataset.speed;
                rate = rate === '1X' ? '倍速' : rate;
                Utils.classList.removeClass(doubleSpeed, 'active');
                if (rate === rateText) {
                    Utils.classList.addClass(doubleSpeed, 'active');
                }
            }
            _this.maskShow();
        });
        var _loop_1 = function (i) {
            doubleSpeeds[i].addEventListener('click', function () {
                var rate = doubleSpeeds[i].dataset.speed;
                var rateText = rate === '1X' ? '倍速' : rate;
                _this.player.speed(rate);
                _this.player.dom.doubleSpeed.innerText = rateText;
                _this.maskhide();
            });
        };
        for (var i = 0; i < doubleSpeeds.length; i++) {
            _loop_1(i);
        }
    };
    // 视频缩略图
    Controller.prototype.initThumbnails = function () {
        var _this = this;
        if (this.player.options.video.thumbnails) {
            this.thumbnails = new Thumbnails({
                container: this.player.dom.playedBarPreview,
                barWidth: this.player.dom.playedBarWrap.offsetWidth,
                url: this.player.options.video.thumbnails,
                events: this.player.events
            });
            this.player.on('loadedmetadata', function () {
                _this.thumbnails.resize(160, (_this.player.video.videoHeight /
                    _this.player.video.videoWidth) *
                    160, _this.player.dom.playedBarWrap.offsetWidth);
            });
        }
    };
    //
    Controller.prototype.initPlayedBar = function () {
        var _this = this;
        var thumbMove = function (e) {
            var clientX = typeof e.clientX !== 'undefined' ? e.clientX : e.changedTouches[0].clientX;
            var percentage = (clientX -
                Utils.getBoundingClientRectViewLeftOrTop(_this.player.dom.playedBarWrap, 'left')) /
                _this.player.dom.playedBarWrap.clientWidth;
            percentage = Math.max(percentage, 0);
            percentage = Math.min(percentage, 1);
            _this.player.bar.set('played', percentage, 'width');
            _this.player.dom.ptime.innerHTML = Utils.secondToTime(percentage * _this.player.video.duration);
        };
        var thumbUp = function (e) {
            var clientX = typeof e.clientX !== 'undefined' ? e.clientX : e.changedTouches[0].clientX;
            document.removeEventListener(Utils.nameMap.dragEnd, thumbUp);
            document.removeEventListener(Utils.nameMap.dragMove, thumbMove);
            var percentage = (clientX -
                Utils.getBoundingClientRectViewLeftOrTop(_this.player.dom.playedBarWrap, 'left')) /
                _this.player.dom.playedBarWrap.clientWidth;
            percentage = Math.max(percentage, 0);
            percentage = Math.min(percentage, 1);
            _this.player.bar.set('played', percentage, 'width');
            _this.player.seek(_this.player.bar.get('played') * _this.player.video.duration);
            _this.player.timer.enable('progress');
        };
        this.player.dom.playedBarWrap.addEventListener(Utils.nameMap.dragStart, function () {
            _this.player.timer.disable('progress');
            document.addEventListener(Utils.nameMap.dragMove, thumbMove);
            document.addEventListener(Utils.nameMap.dragEnd, thumbUp);
        });
        this.player.dom.playedBarWrap.addEventListener(Utils.nameMap.dragMove, function (e) {
            var barWidth = _this.player.dom.playedBarWrap.offsetWidth;
            if (_this.player.video.duration) {
                var px = Utils.cumulativeOffset(_this.player.dom.playedBarWrap).left;
                var clientX = typeof e.clientX !== 'undefined' ? e.clientX : e.changedTouches[0].clientX;
                var tx = clientX - px;
                if (tx < 0 || tx > barWidth) {
                    return;
                }
                var time = _this.player.video.duration * (tx / barWidth);
                if (Utils.isMobile) {
                    _this.thumbnails && _this.thumbnails.show();
                }
                _this.thumbnails && _this.thumbnails.move(tx);
                var BarTimeLeft = 0;
                if (tx - 23 < 0) {
                    BarTimeLeft = 23;
                }
                else if (tx + 23 > barWidth) {
                    BarTimeLeft = barWidth - 23;
                }
                else {
                    BarTimeLeft = tx;
                }
                _this.player.dom.playedBarTime.style.left = BarTimeLeft + "px";
                _this.player.dom.playedBarTime.innerText = Utils.secondToTime(time);
                Utils.classList.removeClass(_this.player.dom.playedBarTime, 'hidden');
                _this._handleThumbnail(tx, time);
            }
        });
        this.player.dom.playedBarWrap.addEventListener(Utils.nameMap.dragEnd, function () {
            if (Utils.isMobile) {
                _this.thumbnails && _this.thumbnails.hide();
            }
        });
        if (!Utils.isMobile) {
            this.player.dom.playedBarWrap.addEventListener('mouseenter', function () {
                if (_this.player.video.duration) {
                    _this.thumbnails && _this.thumbnails.show();
                    Utils.classList.removeClass(_this.player.dom.playedBarTime, 'hidden');
                }
            });
            this.player.dom.playedBarWrap.addEventListener('mouseleave', function () {
                if (_this.player.video.duration) {
                    _this.thumbnails && _this.thumbnails.hide();
                    Utils.classList.addClass(_this.player.dom.playedBarTime, 'hidden');
                }
            });
        }
    };
    Controller.prototype._handleThumbnail = function (tx, time) {
        var halfWidth = 60;
        var barWidth = this.player.dom.playedBarWrap.offsetWidth;
        // const
        var highlights = document.querySelectorAll('.dplayer-highlight');
        for (var i = 0; i < highlights.length; i++) {
            var highlight = highlights[i];
            var highlightImg = highlight.querySelectorAll('.dplayer-highlight-img')[0];
            var styleLeft = (parseFloat(highlight.style.left) / 100) * barWidth;
            var isTrue = tx < halfWidth || tx + halfWidth > barWidth;
            if (!highlightImg) {
                return;
            }
            highlightImg.innerText = Utils.secondToTime(time);
            if (isTrue && styleLeft < halfWidth) {
                highlightImg.style.marginLeft = '-' + styleLeft + 'px';
            }
            if (isTrue && styleLeft > barWidth - halfWidth) {
                highlightImg.style.marginLeft =
                    '-' + (halfWidth - (barWidth - styleLeft) + halfWidth) + 'px';
            }
        }
    };
    // 全屏切换
    Controller.prototype.initFullButton = function () {
        var _this = this;
        this.player.dom.browserFullButton.addEventListener('click', function () {
            _this.player.fullScreen.toggle('browser');
        });
        this.player.dom.webFullButton.addEventListener('click', function () {
            _this.player.fullScreen.toggle('web');
        });
    };
    // 音量
    Controller.prototype.initVolumeButton = function () {
        var _this = this;
        if (!Utils.isMobile) {
            var vHeight_1 = 60;
            var volumeMove_1 = function (event) {
                var e = event || window.event;
                var percentage = 1 -
                    ((e.clientY || e.changedTouches[0].clientY) -
                        Utils.getBoundingClientRectViewLeftOrTop(_this.player.dom.volumeBarWrap, 'top')) /
                        vHeight_1;
                _this.player.volume(percentage);
            };
            var volumeUp_1 = function () {
                document.removeEventListener(Utils.nameMap.dragEnd, volumeUp_1);
                document.removeEventListener(Utils.nameMap.dragMove, volumeMove_1);
            };
            this.player.dom.volumeBarWrapWrap.addEventListener('click', function (event) {
                var e = event || window.event;
                var percentage = 1 -
                    ((e.clientY || e.changedTouches[0].clientY) -
                        Utils.getBoundingClientRectViewLeftOrTop(_this.player.dom.volumeBarWrap, 'top')) /
                        vHeight_1;
                _this.player.volume(percentage);
            });
            this.player.dom.volumeBarWrapWrap.addEventListener(Utils.nameMap.dragStart, function () {
                document.addEventListener(Utils.nameMap.dragMove, volumeMove_1);
                document.addEventListener(Utils.nameMap.dragEnd, volumeUp_1);
            });
            this.player.dom.volumeButton.addEventListener('mouseenter', function () {
                Utils.classList.addClass(_this.player.dom.volumeButton, 'dplayer-volume-active');
            });
            this.player.dom.volumeButton.addEventListener('mouseleave', function () {
                Utils.classList.removeClass(_this.player.dom.volumeButton, 'dplayer-volume-active');
            });
        }
        this.player.dom.volumeButtonIcon.addEventListener('click', function () {
            if (_this.player.video.muted) {
                _this.player.video.muted = false;
                _this.player.switchVolumeIcon();
                _this.player.bar.set('volume', _this.player.volume(), 'height');
            }
            else {
                _this.player.video.muted = true;
                _this.player.dom.volumeIconFont.className = 'iconfont iconvolume-off';
                _this.player.bar.set('volume', 0, 'height');
            }
        });
    };
    Controller.prototype.maskhide = function () {
        Utils.classList.removeClass(this.player.dom.doubleSpeedPopup, 'open');
        Utils.classList.removeClass(this.player.dom.mask, 'dplayer-mask-show');
        this.disableAutoHide = false;
    };
    Controller.prototype.maskShow = function () {
        Utils.classList.addClass(this.player.dom.doubleSpeedPopup, 'open');
        Utils.classList.addClass(this.player.dom.mask, 'dplayer-mask-show');
        this.disableAutoHide = true;
    };
    Controller.prototype.setAutoHide = function () {
        var _this = this;
        // this.show()
        clearTimeout(this.autoHideTimer);
        this.autoHideTimer = window.setTimeout(function () {
            if (!_this.disableAutoHide) {
                _this.hide();
            }
        }, 3000);
    };
    Controller.prototype.show = function () {
        Utils.classList.removeClass(this.player.container, 'dplayer-hide-controller');
    };
    Controller.prototype.hide = function () {
        if (this.player.paused) {
            return;
        }
        if (!Utils.classList.hasClass(this.player.container, 'dplayer-hide-controller')) {
            Utils.classList.addClass(this.player.container, 'dplayer-hide-controller');
        }
    };
    Controller.prototype.isShow = function () {
        return !Utils.classList.hasClass(this.player.container, 'dplayer-hide-controller');
    };
    Controller.prototype.toggle = function () {
        if (this.isShow()) {
            this.hide();
        }
        else {
            this.show();
        }
    };
    Controller.prototype.destroy = function () {
        clearTimeout(this.autoHideTimer);
    };
    return Controller;
}());

var Timer = /** @class */ (function () {
    function Timer(player) {
        this.player = player;
        window.requestAnimationFrame = (function () {
            return window.requestAnimationFrame ||
                window.webkitRequestAnimationFrame ||
                window.mozRequestAnimationFrame ||
                window.oRequestAnimationFrame ||
                window.msRequestAnimationFrame ||
                function (callback) {
                    window.setTimeout(callback, 1000 / 60);
                };
        })();
        this.types = ['loading', 'info', 'fps'];
        this.init();
    }
    Timer.prototype.init = function () {
        var _this = this;
        this.types.map(function (item) {
            if (item !== 'fps') {
                _this["init" + item + "Checker"]();
            }
            return item;
        });
    };
    Timer.prototype.initloadingChecker = function () {
        var _this = this;
        var lastPlayPos = 0;
        var currentPlayPos = 0;
        var bufferingDetected = false;
        this.loadingChecker = window.setInterval(function () {
            if (_this.enableloadingChecker) {
                // whether the video is buffering
                currentPlayPos = _this.player.video.currentTime;
                if (!bufferingDetected &&
                    currentPlayPos === lastPlayPos &&
                    !_this.player.video.paused) {
                    Utils.classList.addClass(_this.player.container, 'dplayer-loading');
                    bufferingDetected = true;
                }
                if (bufferingDetected &&
                    currentPlayPos > lastPlayPos &&
                    !_this.player.video.paused) {
                    Utils.classList.removeClass(_this.player.container, 'dplayer-loading');
                    bufferingDetected = false;
                }
                lastPlayPos = currentPlayPos;
            }
        }, 100);
    };
    // initfpsChecker() {
    //     window.requestAnimationFrame(() => {
    //         if (this.enablefpsChecker) {
    //             this.initfpsChecker();
    //             if (!this.fpsStart) {
    //                 this.fpsStart = new Date();
    //                 this.fpsIndex = 0;
    //             }
    //             else {
    //                 this.fpsIndex++;
    //                 const fpsCurrent = new Date();
    //                 if (fpsCurrent - this.fpsStart > 1000) {
    //                     this.player.infoPanel.fps(this.fpsIndex / (fpsCurrent - this.fpsStart) * 1000);
    //                     this.fpsStart = new Date();
    //                     this.fpsIndex = 0;
    //                 }
    //             }
    //         }
    //         else {
    //             this.fpsStart = 0;
    //             this.fpsIndex = 0;
    //         }
    //     });
    // }
    Timer.prototype.initinfoChecker = function () {
        // this.infoChecker = window.setInterval(() => {
        //     if (this.enableinfoChecker) {
        //         this.player.infoPanel.update();
        //     }
        // }, 1000);
    };
    Timer.prototype.enable = function (type) {
        this["enable" + type + "Checker"] = true;
    };
    Timer.prototype.disable = function (type) {
        this["enable" + type + "Checker"] = false;
    };
    Timer.prototype.destroy = function () {
        var _this = this;
        this.types.map(function (item) {
            _this["enable" + item + "Checker"] = false;
            _this[item + "Checker"] && clearInterval(_this[item + "Checker"]);
            return item;
        });
    };
    return Timer;
}());

var HotKey = /** @class */ (function () {
    function HotKey(player) {
        var _this = this;
        this.video = player.video;
        if (player.options.hotkey) {
            document.addEventListener('keydown', function (e) {
                if (player.focus) {
                    var tag = document.activeElement.tagName.toUpperCase();
                    var editable = document.activeElement.getAttribute('contenteditable');
                    if (tag !== 'INPUT' && tag !== 'TEXTAREA' && editable !== '' && editable !== 'true') {
                        var event_1 = e || window.event;
                        var percentage = void 0;
                        switch (event_1.keyCode) {
                            case 32:
                                event_1.preventDefault();
                                player.toggle();
                                break;
                            case 37:
                                event_1.preventDefault();
                                player.seek(_this.video.currentTime - 15);
                                player.controller.setAutoHide();
                                break;
                            case 39:
                                event_1.preventDefault();
                                player.seek(_this.video.currentTime + 15);
                                player.controller.setAutoHide();
                                break;
                            case 38:
                                event_1.preventDefault();
                                percentage = player.volume() + 0.1;
                                player.volume(percentage);
                                break;
                            case 40:
                                event_1.preventDefault();
                                percentage = player.volume() - 0.1;
                                player.volume(percentage);
                                break;
                        }
                    }
                }
            });
        }
        document.addEventListener('keydown', function (e) {
            var event = e || window.event;
            switch (event.keyCode) {
                case 27:
                    if (player.fullScreen.isFullScreen('web')) {
                        player.fullScreen.cancel('web');
                    }
                    break;
            }
        });
    }
    return HotKey;
}());

var Danmaku = /** @class */ (function () {
    function Danmaku(options) {
        var _this = this;
        this.options = options;
        this.container = this.options.container;
        this.danIndex = 0;
        this.dan = options.data;
        this.showing = true;
        this.paused = true;
        this.events = this.options.events;
        this._measure('');
        window.requestAnimationFrame(function () {
            _this.frame();
        });
    }
    Danmaku.prototype.frame = function () {
        var _this = this;
        if (this.dan.length && !this.paused && this.showing) {
            var item = this.dan[this.danIndex];
            var dan = [];
            while (item && this.options.time() > parseFloat(item.time)) {
                dan.push(item);
                item = this.dan[++this.danIndex];
            }
            this.draw(dan);
        }
        window.requestAnimationFrame(function () {
            _this.frame();
        });
    };
    Danmaku.prototype.draw = function (dan) {
        var _this = this;
        if (this.showing) {
            var itemHeight = this.options.height;
            var danWidth_1 = this.container.offsetWidth;
            var danHeight = this.container.offsetHeight;
            var itemY_1 = parseInt(danHeight / itemHeight + '', 10);
            var danItemRight_1 = function (ele) {
                var eleWidth = ele.offsetWidth || parseInt(ele.style.width, 10);
                var eleRight = ele.getBoundingClientRect().right ||
                    _this.container.getBoundingClientRect().right + eleWidth;
                return _this.container.getBoundingClientRect().right - eleRight;
            };
            var danSpeed_1 = function (width) { return (danWidth_1 + width) / 5; };
            var getTunnel = function (ele, type, width) {
                var tmp = danWidth_1 / danSpeed_1(width);
                var _loop_2 = function (i) {
                    var item = _this.danTunnel[type][i + ''];
                    if (item && item.length) {
                        if (type !== 'right') {
                            return "continue";
                        }
                        for (var j = 0; j < item.length; j++) {
                            var danRight = danItemRight_1(item[j]) - 10;
                            if (danRight <= danWidth_1 - tmp * danSpeed_1(parseInt(item[j].style.width, 10)) ||
                                danRight <= 0) {
                                break;
                            }
                            if (j === item.length - 1) {
                                _this.danTunnel[type][i + ''].push(ele);
                                ele.addEventListener('animationend', function () {
                                    _this.danTunnel[type][i + ''].splice(0, 1);
                                });
                                return { value: i % itemY_1 };
                            }
                        }
                    }
                    else {
                        _this.danTunnel[type][i + ''] = [ele];
                        ele.addEventListener('animationend', function () {
                            _this.danTunnel[type][i + ''].splice(0, 1);
                        });
                        return { value: i % itemY_1 };
                    }
                };
                for (var i = 0; i < itemY_1; i++) {
                    var state_1 = _loop_2(i);
                    if (typeof state_1 === "object")
                        return state_1.value;
                }
                return -1;
            };
            if (Object.prototype.toString.call(dan) !== '[object Array]') {
                dan = [dan];
            }
            var docFragment = document.createDocumentFragment();
            var _loop_1 = function (i) {
                dan[i].type = Utils.number2Type(dan[i].type);
                if (!dan[i].color) {
                    dan[i].color = 16777215;
                }
                var item = document.createElement('div');
                item.classList.add('dplayer-danmaku-item');
                item.classList.add("dplayer-danmaku-" + dan[i].type);
                if (dan[i].border) {
                    item.innerHTML = "<span style=\"border:" + dan[i].border + "\">" + dan[i].text + "</span>";
                }
                else {
                    item.innerHTML = dan[i].text;
                }
                item.style.color = Utils.number2Color(dan[i].color);
                item.addEventListener('animationend', function () {
                    _this.container.removeChild(item);
                });
                var itemWidth = this_1._measure(dan[i].text);
                var tunnel = void 0;
                // adjust
                switch (dan[i].type) {
                    case 'right':
                        tunnel = getTunnel(item, dan[i].type, itemWidth);
                        if (tunnel >= 0) {
                            item.style.width = itemWidth + 1 + 'px';
                            item.style.top = itemHeight * tunnel + 'px';
                            item.style.transform = "translateX(-" + danWidth_1 + "px)";
                        }
                        break;
                    case 'top':
                        tunnel = getTunnel(item, dan[i].type);
                        if (tunnel >= 0) {
                            item.style.top = itemHeight * tunnel + 'px';
                        }
                        break;
                    case 'bottom':
                        tunnel = getTunnel(item, dan[i].type);
                        if (tunnel >= 0) {
                            item.style.bottom = itemHeight * tunnel + 'px';
                        }
                        break;
                    default:
                        console.error("Can't handled danmaku type: " + dan[i].type);
                }
                if (tunnel >= 0) {
                    // move
                    item.classList.add('dplayer-danmaku-move');
                    // insert
                    docFragment.appendChild(item);
                }
            };
            var this_1 = this;
            for (var i = 0; i < dan.length; i++) {
                _loop_1(i);
            }
            this.container.appendChild(docFragment);
            return docFragment;
        }
    };
    Danmaku.prototype.seek = function () {
        this.clear();
        for (var i = 0; i < this.dan.length; i++) {
            if (this.dan[i].time >= this.options.time()) {
                this.danIndex = i;
                break;
            }
            this.danIndex = this.dan.length;
        }
    };
    Danmaku.prototype.clear = function () {
        this.danTunnel = {
            right: {},
            top: {},
            bottom: {}
        };
        this.danIndex = 0;
        this.options.container.innerHTML = '';
        this.events && this.events.trigger('danmaku_clear');
    };
    Danmaku.prototype.play = function () {
        this.paused = false;
    };
    Danmaku.prototype.pause = function () {
        this.paused = true;
    };
    Danmaku.prototype.resize = function () {
        var danWidth = this.container.offsetWidth;
        var items = this.container.getElementsByClassName('dplayer-danmaku-item');
        for (var i = 0; i < items.length; i++) {
            items[i].style.transform = "translateX(-" + danWidth + "px)";
        }
    };
    Danmaku.prototype.hide = function () {
        this.showing = false;
        this.pause();
        this.clear();
        this.events && this.events.trigger('danmaku_hide');
    };
    Danmaku.prototype.show = function () {
        this.seek();
        this.showing = true;
        this.play();
        this.events && this.events.trigger('danmaku_show');
    };
    Danmaku.prototype._measure = function (text) {
        if (!this.context) {
            var measureStyle = getComputedStyle(this.container.getElementsByClassName('dplayer-danmaku-item')[0], null);
            this.context = document.createElement('canvas').getContext('2d');
            this.context.font = measureStyle.getPropertyValue('font');
        }
        return this.context.measureText(text).width;
    };
    return Danmaku;
}());

/*
 * @Author: your name
 * @Date: 2020-04-07 18:06:30
 * @LastEditTime: 2020-05-21 17:01:00
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: \FancyPlayer\src\core\section.ts
 */
var Section = /** @class */ (function () {
    function Section(player) {
        this.player = player;
        this.startMouseDownXY = {
            x: 0,
            y: 0
        };
        this.sectionDoms = [];
        this.cacheY = 0;
        this.currentY = 0;
        this.isPress = false;
        this.isDrag = false;
        this.section = this.player.dom.section;
        this.sectionInner = this.player.dom.sectionInner;
        this.initSections();
        this.sectionInner.addEventListener(Utils.nameMap.dragStart, this.onMouseDown.bind(this));
    }
    Object.defineProperty(Section, "SLIDE_MIN", {
        get: function () {
            return 3;
        },
        enumerable: true,
        configurable: true
    });
    Section.prototype.resetValue = function () {
        this.sectionDoms = [];
        this.cacheY = 0;
        this.currentY = 0;
        this.isPress = false;
        this.isDrag = false;
    };
    Section.prototype.getViewPortHeight = function () {
        return document.documentElement.clientHeight || document.body.clientHeight;
    };
    Section.prototype.getMaxDistance = function () {
        return 0;
    };
    Section.prototype.getMinDistance = function () {
        var sectionHeight = this.section.clientHeight;
        var sectionInnerHeight = this.sectionInner.clientHeight;
        var minDistance = -(sectionInnerHeight - sectionHeight);
        return minDistance;
    };
    Section.prototype.controlAlignCenter = function () {
        if (this.getMinDistance() > 0) {
            Utils.classList.addClass(this.player.dom.section, 'align-center');
        }
        else {
            Utils.classList.removeClass(this.player.dom.section, 'align-center');
        }
    };
    Section.prototype.sectionsClick = function (data) {
        if (this.isDrag) {
            return;
        }
        this.nodesActive_(data.index);
        this.player.events.trigger('node', data);
    };
    Section.prototype.onMouseDown = function (e) {
        if (e.type === 'touchstart') {
            e.clientX = e.touches[0].clientX;
            e.clientY = e.touches[0].clientY;
        }
        this.startMouseDownXY = {
            x: e.clientX,
            y: e.clientY
        };
        this.currentY = this.cacheY;
        this.isPress = true;
        document.addEventListener(Utils.nameMap.dragMove, this.onDragging.bind(this));
        document.addEventListener(Utils.nameMap.dragEnd, this.onDragEnd.bind(this));
    };
    Section.prototype.onDragging = function (e) {
        if (!this.isPress) {
            return;
        }
        if (e.type === 'touchmove') {
            e.clientY = e.touches[0].clientY;
        }
        var clientX = e.clientX;
        var clientY = e.clientY;
        var startMouseDownX = this.startMouseDownXY.x;
        var startMouseDownY = this.startMouseDownXY.y;
        var currentY = this.currentY;
        var minDistance = this.getMinDistance();
        var maxDistance = this.getMaxDistance();
        if (minDistance > 0) {
            return;
        }
        if (Math.abs(clientX - startMouseDownX) > Section.SLIDE_MIN ||
            Math.abs(clientY - startMouseDownY) > Section.SLIDE_MIN) {
            this.isDrag = true;
        }
        var distance = clientY - startMouseDownY + currentY;
        this.translate(distance);
    };
    Section.prototype.onDragEnd = function (e) {
        var _this = this;
        setTimeout(function () {
            if (_this.isPress) {
                _this.isPress = false;
                _this.isDrag = false;
            }
        });
        document.removeEventListener(Utils.nameMap.dragMove, this.onDragging.bind(this));
        document.removeEventListener(Utils.nameMap.dragEnd, this.onDragEnd.bind(this));
    };
    Section.prototype.nodesActive_ = function (index, colour) {
        colour = colour || '#ddd';
        for (var i = 0; i < this.sectionDoms.length; i++) {
            if (i <= index) {
                this.sectionDoms[i].style.color = colour;
            }
            else {
                this.sectionDoms[i].style.color = '#fff';
            }
        }
        this.followPlaySlide(this.sectionDoms[index]);
    };
    Section.prototype.followPlaySlide = function (element) {
        var boxOffsetTop = this.section.getBoundingClientRect().top;
        var boxOffsetHeight = this.section.getBoundingClientRect().height;
        var itemOffsetTop = element.getBoundingClientRect().top;
        var itemOffsetHeight = element.getBoundingClientRect().height;
        var topDifference = boxOffsetTop - itemOffsetTop;
        var difference = 0;
        if (topDifference > 0) {
            difference = topDifference + this.cacheY;
            this.translate(difference);
        }
        else if (itemOffsetTop + itemOffsetHeight > boxOffsetTop + boxOffsetHeight) {
            difference =
                -(itemOffsetTop + itemOffsetHeight - (boxOffsetTop + boxOffsetHeight)) +
                    this.cacheY +
                    -itemOffsetHeight;
            this.translate(difference);
        }
    };
    Section.prototype.translate = function (distance) {
        var minDistance = this.getMinDistance();
        var maxDistance = this.getMaxDistance();
        distance = distance > maxDistance ? maxDistance : distance;
        distance = distance < minDistance ? minDistance : distance;
        this.cacheY = distance;
        this.sectionInner.style.transform = "translateY(" + distance + "px)";
    };
    // 侧边节点
    Section.prototype.initSections = function () {
        var _this = this;
        if (this.player.options.isTimeNode) {
            this.player.on('durationchange', function () {
                var duration = _this.player.video.duration;
                if (duration !== 1 && duration !== 0 && duration !== Infinity) {
                    _this._createSectionDom();
                }
            });
        }
        else {
            this._createSectionDom();
        }
    };
    Section.prototype._createSectionDom = function () {
        var highlightOptions = this.player.options.highlight;
        this.sectionDoms = [];
        this.sectionInner.innerHTML = '';
        if (highlightOptions) {
            for (var i = 0; i < highlightOptions.length; i++) {
                if (highlightOptions[i].nodeName) {
                    var highlightNode = document.createElement('div');
                    var highlightNodeText = document.createElement('span');
                    var highlightNodeIco = document.createElement('span');
                    var highlightNodeBor = document.createElement('span');
                    var time = highlightOptions[i].time;
                    highlightNodeBor.className = 'dplayer-section-line';
                    highlightNode.className = 'dplayer-section-item';
                    highlightNodeText.innerHTML = highlightOptions[i].nodeName;
                    if (time) {
                        highlightNode.setAttribute('data-time', time + '');
                    }
                    if (this.getViewPortHeight() <= 768) {
                        Utils.classList.addClass(highlightNode, 'small-space');
                    }
                    if (highlightOptions[i].fileType === 1) {
                        highlightNodeIco.className = 'iconfont iconvideo';
                    }
                    else if (highlightOptions[i].fileType === 2) {
                        highlightNodeIco.className = 'iconfont iconphoto';
                    }
                    highlightNode.appendChild(highlightNodeIco);
                    highlightNode.appendChild(highlightNodeText);
                    highlightNode.appendChild(highlightNodeBor);
                    this.sectionInner.appendChild(highlightNode);
                    this.sectionDoms.push(highlightNode);
                    highlightNode.addEventListener('click', this.sectionsClick.bind(this, {
                        index: i,
                        ele: highlightNode,
                        nodes: this.sectionDoms
                    }));
                }
            }
        }
    };
    // 自定义进度条提示点
    Section.prototype.setHighlights = function (highlightOptions) {
        var _this = this;
        var duration = this.player.video.duration;
        if (duration !== 1 && duration !== 0 && duration !== Infinity) {
            if (highlightOptions) {
                var highlights = document.querySelectorAll('.dplayer-highlight');
                [].slice.call(highlights, 0).forEach(function (item) {
                    _this.player.dom.playedBarWrap.removeChild(item);
                });
                for (var i = 0; i < highlightOptions.length; i++) {
                    var highlightNode = document.createElement('div');
                    Utils.classList.addClasses(highlightNode, 'dplayer-highlight node-large');
                    highlightNode.style.borderColor = this.player.options.theme;
                    highlightNode.style.left = (highlightOptions[i].time / duration) * 100 + '%';
                    if (highlightOptions[i].thumbnail) {
                        highlightNode.innerHTML = "<div class=\"dplayer-highlight-img\" style=\"background-image: url('" + highlightOptions[i].thumbnail + "');\"></div>";
                    }
                    this.player.dom.playedBarWrap.insertBefore(highlightNode, this.player.dom.playedBarTime);
                }
            }
        }
    };
    return Section;
}());

var instances = [];
var Player = /** @class */ (function () {
    function Player(options) {
        var _this = this;
        this.options = handleOption(options);
        if (this.options.video.quality) {
            this.qualityIndex = this.options.video.defaultQuality;
            this.qualityVideo = this.options.video.quality[this.qualityIndex];
        }
        this.tran = new I18n(this.options.lang).tran;
        this.events = new Events();
        this.user = new User(this);
        this.container = this.options.container;
        Utils.classList.addClasses(this.container, 'dplayer dplayer-hide-controller');
        if (!this.options.danmaku) {
            Utils.classList.addClass(this.container, 'dplayer-no-danmaku');
        }
        if (this.options.live) {
            Utils.classList.addClass(this.container, 'dplayer-live');
        }
        if (Utils.isMobile) {
            Utils.classList.addClass(this.container, 'dplayer-mobile');
        }
        this.arrow = this.container.offsetWidth <= 500;
        if (this.arrow) {
            Utils.classList.addClass(this.container, 'dplayer-arrow');
        }
        this.dom = new Dom(this.container, options, this.tran);
        this.video = this.dom.video;
        this.bar = new Bar(this.dom);
        this.hotkey = new HotKey(this);
        this.fullScreen = new FullScreen(this);
        this.controller = new Controller(this);
        if (this.options.highlight && this.options.highlight.length) {
            this.section = new Section(this);
        }
        if (this.options.danmaku) {
            this.danmaku = new Danmaku({
                container: this.dom.danmaku,
                height: this.arrow ? 24 : 30,
                data: [
                    { time: 0, type: 0, color: 16777215, author: 'f381ae18', text: '我的天怎么被顶上来了' },
                    { time: 0, type: 0, color: 16777215, author: '832afb47', text: '不错' },
                    {
                        time: 0.392,
                        type: 0,
                        color: 16777215,
                        author: 'bd364cdf',
                        text: '我这首歌至少听了100遍了！！！！'
                    },
                    {
                        time: 0.439,
                        type: 0,
                        color: 6830715,
                        author: '90f10f7f',
                        text: '此生无悔入四月，来生愿做友人A。'
                    },
                    { time: 0.835, type: 0, color: 16777215, author: 'c3f20575', text: '朱军，拔剑吧' },
                    { time: 1.146, type: 0, color: 16777215, author: '6739cb0f', text: '弹幕清了？' },
                    { time: 1.198, type: 0, color: 16777215, author: '63d9b55d', text: '果然最爱鹅屋了' }
                ],
                time: function () { return _this.video.currentTime; },
                events: this.events
            });
        }
        document.addEventListener('click', function () {
            _this.focus = false;
        }, true);
        this.container.addEventListener('click', function () {
            _this.focus = true;
        }, true);
        this.paused = true;
        this.timer = new Timer(this);
        this.initVideo(this.video, this.options.video.type);
        if (!this.danmaku && this.options.autoplay) {
            this.play();
        }
        instances.push(this);
    }
    Player.prototype.initMSE = function (video, type) {
        this.type = type;
        if (this.options.video.customType && this.options.video.customType[type]) {
            if (Object.prototype.toString.call(this.options.video.customType[type]) === '[object Function]') {
                this.options.video.customType[type](this.video, this);
            }
            else {
                console.error("Illegal customType: " + type);
            }
        }
        else {
            if (this.type === 'auto') {
                if (/m3u8(#|\?|$)/i.exec(video.src)) {
                    this.type = 'hls';
                }
                else if (/.flv(#|\?|$)/i.exec(video.src)) {
                    this.type = 'flv';
                }
                else if (/.mpd(#|\?|$)/i.exec(video.src)) {
                    this.type = 'dash';
                }
                else {
                    this.type = 'normal';
                }
            }
            if (this.type === 'hls' &&
                (video.canPlayType('application/x-mpegURL') ||
                    video.canPlayType('application/vnd.apple.mpegURL'))) {
                this.type = 'normal';
            }
        }
    };
    Player.prototype.initVideo = function (video, type) {
        var _this = this;
        this.initMSE(video, type);
        /**
         * video events
         */
        // show video time: the metadata has loaded or changed
        this.on('durationchange', function () {
            // compatibility: Android browsers will output 1 or Infinity at first
            if (video.duration !== 1 && video.duration !== Infinity) {
                _this.dom.dtime.innerHTML = Utils.secondToTime(video.duration);
            }
        });
        // show video loaded bar: to inform interested parties of progress downloading the media
        this.on('progress', function () {
            var percentage = video.buffered.length
                ? video.buffered.end(video.buffered.length - 1) / video.duration
                : 0;
            _this.bar.set('loaded', percentage, 'width');
        });
        // video download error: an error occurs
        this.on('error', function () {
            if (!_this.video.error) {
                // Not a video load error, may be poster load failed, see #307
                return;
            }
            // this.tran && this.notice && this.type !== 'webtorrent' & this.notice(this.tran('Video load failed'), -1);
        });
        // video end
        this.on('ended', function () {
            _this.bar.set('played', 1, 'width');
            // if (!this.setting.loop) {
            //   this.pause();
            // }
            // else {
            //   this.seek(0);
            //   this.play();
            // }
            if (_this.danmaku) {
                _this.danmaku.danIndex = 0;
            }
        });
        this.on('play', function () {
            if (_this.paused) {
                _this.play();
            }
        });
        this.on('pause', function () {
            if (!_this.paused) {
                _this.pause();
            }
        });
        this.on('timeupdate', function () {
            _this.bar.set('played', _this.video.currentTime / _this.video.duration, 'width');
            var currentTime = Utils.secondToTime(_this.video.currentTime);
            if (_this.dom.ptime.innerHTML !== currentTime) {
                _this.dom.ptime.innerHTML = currentTime;
            }
        });
        var _loop_1 = function (i) {
            video.addEventListener(this_1.events.videoEvents[i], function () {
                _this.events.trigger(_this.events.videoEvents[i]);
            });
        };
        var this_1 = this;
        for (var i = 0; i < this.events.videoEvents.length; i++) {
            _loop_1(i);
        }
        this.volume(this.user.get('volume'), true, true);
        // if (this.options.subtitle) {
        //   this.subtitle = new Subtitle(this.template.subtitle, this.video, this.options.subtitle, this.events);
        //   if (!this.user.get('subtitle')) {
        //     this.subtitle.hide();
        //   }
        // }
    };
    Player.prototype.speed = function (rate) {
        this.video.playbackRate = parseFloat(rate);
    };
    Player.prototype.on = function (name, callback) {
        this.events.on(name, callback);
    };
    Player.prototype.resize = function () {
        if (this.danmaku) {
            this.danmaku.resize();
        }
        if (this.controller.thumbnails) {
            this.controller.thumbnails.resize(160, (this.video.videoHeight / this.video.videoWidth) * 160, this.dom.playedBarWrap.offsetWidth);
        }
        this.events.trigger('resize');
    };
    Player.prototype.toggle = function () {
        if (this.video.paused) {
            this.play();
        }
        else {
            this.pause();
        }
    };
    Player.prototype.play = function () {
        var _this = this;
        this.paused = false;
        Utils.classList.removeClass(this.dom.bezelIconBox, 'dplayer-bezel-pause');
        this.dom.playButtonIco.className = 'iconfont iconpause';
        var playedPromise = Promise.resolve(this.video.play());
        playedPromise.catch(function () {
            _this.pause();
        });
        this.timer.enable('loading');
        Utils.classList.removeClass(this.container, 'dplayer-paused');
        Utils.classList.addClass(this.container, 'dplayer-playing');
        if (this.danmaku) {
            this.danmaku.play();
        }
        if (this.options.mutex) {
            for (var i = 0; i < instances.length; i++) {
                if (this !== instances[i]) {
                    instances[i].pause();
                }
            }
        }
    };
    /**
     * Pause video
     */
    Player.prototype.pause = function () {
        this.paused = true;
        Utils.classList.removeClass(this.container, 'dplayer-loading');
        Utils.classList.addClass(this.dom.bezelIconBox, 'dplayer-bezel-pause');
        this.dom.playButtonIco.className = 'iconfont iconplay';
        this.video.pause();
        this.timer.disable('loading');
        Utils.classList.removeClass(this.container, 'dplayer-playing');
        Utils.classList.addClass(this.container, 'dplayer-paused');
        if (this.danmaku) {
            this.danmaku.pause();
        }
    };
    Player.prototype.volume = function (percentage, nostorage, nonotice) {
        var isOff = false;
        if (!percentage) {
            isOff = true;
        }
        // percentage = parseFloat(percentage!);
        if (!isNaN(percentage)) {
            percentage = Math.max(percentage, 0);
            percentage = Math.min(percentage, 1);
            this.bar.set('volume', percentage, 'height');
            var formatPercentage = (percentage * 100).toFixed(0) + "%";
            this.dom.volumeBarWrapWrap.dataset.balloon = formatPercentage;
            if (!nostorage) {
                this.user.set('volume', percentage);
            }
            if (!nonotice) {
                this.notice(this.tran('Volume') + " " + (percentage * 100).toFixed(0) + "%");
            }
            this.video.volume = percentage;
            // this.video.volume =  percentage
            if (this.video.muted) {
                this.video.muted = false;
            }
            this.switchVolumeIcon();
        }
        if (isOff) {
            return Number(this.video.volume);
        }
        return percentage;
    };
    Player.prototype.switchVolumeIcon = function () {
        if (this.volume() >= 0.95) {
            this.dom.volumeIconFont.className = 'iconfont iconvolume-up';
        }
        else if (this.volume() > 0) {
            this.dom.volumeIconFont.className = 'iconfont iconvolume-down';
        }
        else {
            this.dom.volumeIconFont.className = 'iconfont iconvolume-off';
        }
    };
    Player.prototype.notice = function (text, time, opacity) {
        var _this = this;
        if (time === void 0) { time = 2000; }
        if (opacity === void 0) { opacity = 0.8; }
        this.dom.notice.innerHTML = text;
        this.dom.notice.style.opacity = opacity;
        if (this.noticeTime) {
            clearTimeout(this.noticeTime);
        }
        this.events.trigger('notice_show', text);
        if (time > 0) {
            this.noticeTime = setTimeout(function () {
                _this.dom.notice.style.opacity = 0;
                _this.events.trigger('notice_hide');
            }, time);
        }
    };
    Player.prototype.seek = function (time) {
        time = Math.max(time, 0);
        if (this.video.duration) {
            time = Math.min(time, this.video.duration);
        }
        if (this.video.currentTime < time) {
            this.notice(this.tran('FF') + " " + (time - this.video.currentTime).toFixed(0) + " " + this.tran('s'));
        }
        else if (this.video.currentTime > time) {
            this.notice(this.tran('REW') + " " + (this.video.currentTime - time).toFixed(0) + " " + this.tran('s'));
        }
        this.video.currentTime = time;
        if (this.danmaku) {
            this.danmaku.seek();
        }
        this.bar.set('played', time / this.video.duration, 'width');
        this.dom.ptime.innerHTML = Utils.secondToTime(time);
    };
    Player.prototype.switchVideo = function (video, highlight) {
        this.pause();
        this.video.poster = video.pic ? video.pic : '';
        this.video.src = video.url;
        this.initMSE(this.video, video.type || 'auto');
        if (highlight) {
            this.options.highlight = highlight;
            if (!this.section) {
                this.section = new Section(this);
            }
            else {
                this.section.resetValue();
                this.section.initSections();
            }
        }
    };
    Player.prototype.destroy = function () {
        instances.splice(instances.indexOf(this), 1);
        this.pause();
        this.controller.destroy();
        this.timer.destroy();
        this.video.src = '';
        this.container.innerHTML = '';
        this.events.trigger('destroy');
    };
    return Player;
}());

/*
 * @Author: your name
 * @Date: 2020-05-20 16:56:20
 * @LastEditTime: 2020-05-25 11:53:01
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: \vue-dplayer-master\src\index.js
 */
var VueDPlayer = {
  props: {
    options: {
      type: Object
    },
    haveData: {
      type: Boolean
    }
  },
  watch: {
    haveData: function (value) {
      if (value) {
        this.initPlayer();
      }
    }
  },
  data: function data() {
    return {
      dp: null,
      currentIndex: 0,
      videoData: {},
      isFullScreen: false,
      fullScreenType: "web",
      qiniuUrl: 'http://qiniu.tongyishidai.com/',
      imageUrl: '',
      nodes: []
    };
  },
  mounted: function mounted() {
    this.createImgDom();
  },
  methods: {
    initPlayer: function initPlayer() {
      var this$1 = this;

      var highlight = this.options.highlight;
      if (highlight && highlight.length) {
        if (highlight[0].fileType === 1) {
          this.options.video.url = this.qiniuUrl + highlight[0].fileUrl;
          this.options.video.pic = this.qiniuUrl + highlight[0].fileUrl + '?vframe/jpg/offset/0';
        } else if (highlight[0].fileType === 2) {
          this.addClass(this.boxDom, 'show');
          this.imageUrl = this.qiniuUrl + highlight[0].fileUrl;
          this.imgDom.setAttribute('src', this.imageUrl);
        }
      }
      this.options.container = this.$el;
      var player = this.dp = new Player(this.options);
      console.log(this.options);
      var events = player.events;
      this.navigateRight.style.color = player.options.theme;
      this.navigateLeft.style.color = player.options.theme;
      Object.keys(events).forEach(function (item) {
        if (item === 'events') {
          return false;
        } else {
          events[item].forEach(function (event) {
            player.on(event, function () { return this$1.$emit(event); });
          });
        }
      });
      if (!highlight || !highlight.length) {
        return;
      }
      this.handleChangeNode(this.currentIndex);
      player.on('ended', function () {
        if (this$1.currentIndex < highlight.length - 1) {
          this$1.handleChangeNode(this$1.currentIndex + 1);
        }
      });

      player.on("node", function (data) {
        this$1.handleChangeNode(data.index);
      });
      player.on("fullscreen", function () {
        this$1.isFullScreen = true;
        this$1.fullScreenType = "browser";
      });
      player.on("webfullscreen", function () {
        this$1.isFullScreen = true;
        this$1.fullScreenType = "web";
      });
      player.on("webfullscreen_cancel", function () {
        this$1.isFullScreen = false;
      });
      player.on("fullscreen_cancel", function () {
        this$1.isFullScreen = false;
      });
      player.on("durationchange", function () {
        player.section.setHighlights(this$1.nodes);
      });
      player.on("timeupdate", function () {
        var currentTime = Math.floor(player.dom.video.currentTime);
        if (this$1.currentTime !== currentTime) {
          this$1.currentTime = currentTime;
          if (this$1.nodes) {
            for (var i = 0; i < this$1.nodes.length; i++) {
              console.log(currentTime, this$1.nodes[i].time);
              if (currentTime === this$1.nodes[i].time) {
                player.pause();
              }
            }
          }
        }
      });
    },
    createImgDom: function createImgDom() {
      var this$1 = this;

      var boxDom = this.boxDom = document.createElement('div');
      var imgDom = this.imgDom = document.createElement('img');
      var navigateRight = this.navigateRight = document.createElement('button');
      var navigateLeft = this.navigateLeft = document.createElement('button');
      var arrowRight = document.createElement('div');
      var arrowLeft = document.createElement('div');
      navigateRight.className = 'dplayer-navigate-right';
      navigateLeft.className = 'dplayer-navigate-left';
      arrowRight.className = 'dplayer-controls-arrow';
      arrowLeft.className = 'dplayer-controls-arrow';
      boxDom.className = 'dplayer-image-box';
      imgDom.className = 'dplayer-image';
      imgDom.setAttribute('preview', 1);
      imgDom.setAttribute('src', this.imageUrl);
      navigateRight.appendChild(arrowRight);
      navigateLeft.appendChild(arrowLeft);
      boxDom.appendChild(imgDom);
      boxDom.appendChild(navigateRight);
      boxDom.appendChild(navigateLeft);
      navigateRight.addEventListener('click', function () {
        this$1.handleChangeNode(this$1.currentIndex + 1);
      });
      navigateLeft.addEventListener('click', function () {
        this$1.handleChangeNode(this$1.currentIndex - 1);
      });
      this.$el.appendChild(boxDom);
    },
    formatTime: function formatTime(time) {
      var date = new Date(time);
      var hours = date.getHours();
      var minutes = date.getMinutes();
      var seconds = date.getSeconds();
      return 3600 * hours + 60 * minutes + seconds;
    },
    handleChangeNode: function handleChangeNode(index) {
      var this$1 = this;

      var highlight = this.options.highlight;
      if (index < 0 || index >= highlight.length) {
        return;
      }
      this.currentIndex = index;
      this.nodes = [];
      if (highlight[index].fileType === 1) {
        if (this.dp.section.sectionDoms.length) {
          if (highlight[index].nodes && highlight[index].nodes.length) {
            for (var i = 0; i < highlight[index].nodes.length; i++) {
              var nodes = {
                guid: highlight[index].nodes[i].guid,
                time: highlight[index].nodes[i].time
              };
              this$1.nodes.push(nodes);
              this$1.nodes[i].time = this$1.formatTime(this$1.nodes[i].time);
            }
            this.nodes = this.nodes.filter(function (item) { return item.time; });
          }
          this.dp.section.nodesActive_(index, this.dp.options.theme);
          this.videoData.url = this.qiniuUrl + highlight[index].fileUrl;
          this.videoData.pic = this.qiniuUrl + highlight[index].fileUrl + "?vframe/jpg/offset/0";
          this.dp.switchVideo(this.videoData);
          this.dp.play();
          this.removeClass(this.boxDom, 'show');
        }
      } else {
        if (this.isFullScreen) {
          this.dp.fullScreen.toggle(this.fullScreenType);
        }
        if (!this.dp.paused) {
          this.dp.pause();
        }
        if (index === 0) {
          this.addClass(this.navigateLeft, 'lucency');
          this.removeClass(this.navigateRight, 'lucency');
        } else if (index === highlight.length - 1) {
          this.removeClass(this.navigateLeft, 'lucency');
          this.addClass(this.navigateRight, 'lucency');
        } else {
          this.removeClass(this.navigateRight, 'lucency');
          this.removeClass(this.navigateLeft, 'lucency');
        }
        this.addClass(this.boxDom, 'show');
        this.imageUrl = this.qiniuUrl + highlight[index].fileUrl;
        this.imgDom.setAttribute('src', this.imageUrl);
      }
    },
    addClass: function addClass(element, className) {
      if (this.hasClass(element, className)) {
        return;
      }
      var newClass = element.className.split(' ');
      newClass.push(className);
      element.className = newClass.join(' ');
    },
    removeClass: function removeClass(element, className) {
      if (this.hasClass(element, className)) {
        var reg = new RegExp('(\\s|^)' + className + '(\\s|$)');
        element.className = element.className.replace(reg, '');
      }
    },
    hasClass: function hasClass(element, className) {
      var reg = new RegExp('(^|\\s)' + className + '(\\s|$)');
      return reg.test(element.className);
    }
  },
  install: function install(Vue, ref) {
    if ( ref === void 0 ) ref = {};
    var name = ref.name; if ( name === void 0 ) name = 'd-player';

    Vue.component(name, this);
  },
  render: function render(h) {
    return h('div', {
      class: 'dplayer'
    }, []);
  }
};

if (typeof window !== 'undefined' && window.Vue) {
  window.VueDPlayer = VueDPlayer;
}

return VueDPlayer;

})));
